local json = require("json")
local const = require("const")
local utils = require("utils3")
local format = string.format

utils.check_request_method("POST")
utils.check_auth()
utils.check_permission("SceneManage")

local function disable_scene(id)
    local ret, err, cause = utils.dbus_call_edge(
            const.scene.wkn, const.scene.obj, const.scene.inf,
            const.scene.method.disable_scene, id)
    if not ret then
        return false, err, format("failed to disable scene(%s): %s", id, cause)
    end

    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return false, const.MESSAGE_INVALIDREPLY, "not a JSON object"
    end
    if res["code"] ~= 0 then
        -- Keep consistent with Scene error definition.
        local code = const.MESSAGE_FAILEDREPLY
        local message = res["message"]
        if res["code"] == 5 then
            code = const.SERVICE_SCENE_NOT_EXIST
        elseif res["code"] == 4 then
            code = const.SERVICE_SCENE_NOT_AT_LEVEL0
        else
            message = { code = res["code"], message = res["message"] }
        end
        return false, code, message
    end
    return true
end

local id = ngx.var[1]
local ok, err, cause = disable_scene(id)
if not ok then
    return utils.error_exit(ngx.HTTP_INTERNAL_SERVER_ERROR, err, cause)
end

ngx.header['Content-Type'] = 'application/json; charset=utf-8'
ngx.exit(ngx.HTTP_OK)