local json = require("json")
local utils = require("utils")
local const = require("const")
local request_method = ngx.var.request_method
local max_rule_count = 30

utils.check_login()
_, login_token = utils.check_permission("MessageRoute")

local function check_param_is_null(name, value)
    
    if value == nil or type(value) ~= "string" then
        local ret = name.." must not be null"
        utils.error_exit(400, ret)
    end
end

local function check_sourcetype(source)
    
    if source == "device" or source == "function" or source == "IotHub" then
        return true
    else
        utils.error_exit(400, "current version only support device/function/Iothub ")
    end
end

local function check_targettype(target)
    if target == "function" or target == "IotHub" or target == "StreamCompute" then
        return true
    else
        utils.error_exit(400, "current version only support router to streamcompte/iothub/function")
    end
end

if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local sourceType = params["SourceType"]
    local sourceData = params["SourceData"]
    local topicFilter = params["TopicFilter"]
    local targetType = params["TargetType"]
    local targetData = params["TargetData"]
    
    check_param_is_null("sourceType", sourceType)
    check_param_is_null("topicFilter", topicFilter)
    check_param_is_null("targetType", targetType)
    if sourceType ~= "IotHub" then
        check_param_is_null("sourceData", sourceData)
    end
    check_sourcetype(sourceType)
    if sourceType ~= "IotHub" then
        check_param_is_null("targetData", targetData)
    end
    check_targettype(targetType)

    if sourceType == "device" then
        if not (topicFilter == "property" or topicFilter == "all") then
            utils.error_exit(400, "topic filter should be property/all when source type is device")
        end
    end

    local cmd = nil
    if targetType == "StreamCompute" then
        cmd = string.format(const.subscribe.fmt_sub_raw, sourceType, sourceData, topicFilter, "iot.edge."..targetData)    
    elseif targetType == "IotHub" then
        cmd = string.format(const.subscribe.fmt_sub_raw, sourceType, sourceData, topicFilter, "IotHub")    
    else
        utils.error_exit(500, json.encode("current version only support router to stream compte"))
    end

    local ok, msg_raw, err = utils.dbus_call_edge(const.subscribe.wkn,
                                        const.subscribe.obj,
                                        const.subscribe.inf,
                                        const.subscribe.interface_read_list)
    if (ok == "ok" and msg_raw) then
        local msg_raw_json = json.decode(msg_raw[1])
        local content = msg_raw_json["content"]
        
        if #content >= max_rule_count then
            utils.error_exit(400, "rule count already > "..max_rule_count)
        end
    end 

    ok, msg_raw, err = utils.dbus_call_edge(const.subscribe.wkn,
                                        const.subscribe.obj,
                                        const.subscribe.inf,
                                        const.subscribe.interface_sub,
                                        cmd)

    local msg_ret = {}
    if (ok and msg_raw) then
        msg_ret["success"] = "ok"
    else
        utils.error_exit(500, json.encode("failed to query: "..err))
    end 

    utils.error_exit(200, json.encode(msg_ret)) 

end
utils.error_exit(400, "unsupported request method")

