--- List drivers that support upgrade.

local const = require("const")
local json = require("json")
local utils = require("utils")
local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("OTAManage")

if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
utils_header.check_public_body_data()

local function list_ota_drivers()
    local ok, ret, err = utils.dbus_call_edge(
            const.device_ota.wkn,
            const.device_ota.obj,
            const.device_ota.inf,
            const.device_ota.method.list_drivers)
    if not ok then
        return nil, err
    end
    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, "response not a JSON object"
    end
    if res["code"] ~= 0 then
        return nil, tostring(res["message"])
    end
    local data = res["data"]
    if not data or type(data) ~= "table" then
        return nil, "data in response not a array"
    end
    return res["data"]
end

local drivers, err = list_ota_drivers()
if drivers == nil then
    return utils.error_exit(500, err)
end

local resp = {
    code = 200,
    message = "success",
    data = { List = {} }
}
for _, v in ipairs(drivers) do
    local item = {
        DriverId = v["driverId"],
        DriverName = v["driverName"],
    }
    table.insert(resp.data.List, item)
end
ngx.say(json.encode(resp))
