--- Enable a specified scene.

local const = require("const")
local json = require("json")
local utils = require("utils")
local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("SceneManage")

if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()
local scene_id = params["SceneId"]
if type(scene_id) ~= "string" or #scene_id == 0 then
    return utils.error_exit(400, "SceneId is not string or is empty")
end

local function disable_scene(id)
    local ok, ret, err = utils.dbus_call_edge(
            const.scene.wkn, const.scene.obj, const.scene.inf,
            const.scene.method.disable_scene, id)
    if not ok then
        return nil, err
    end
    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, "response not a JSON object"
    end
    if res["code"] ~= 0 then
        return nil, tostring(res["message"])
    end
    return true
end

local ok, err = disable_scene(scene_id)
if not ok then
    return utils.error_exit(500, err)
end

local resp = {
    code = 200,
    message = "success",
}
ngx.say(json.encode(resp))
