--- List scenes.

local const = require("const")
local json = require("json")
local utils = require("utils")
local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("SceneManage")

if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
utils_header.check_public_body_data()

local function list_scenes()
    local ok, ret, err = utils.dbus_call_edge(
            const.scene.wkn, const.scene.obj, const.scene.inf,
            const.scene.method.list_scene_stats)
    if not ok then
        return nil, err
    end
    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, "response not a JSON object"
    end
    if res["code"] ~= 0 then
        return nil, tostring(res["message"])
    end
    local data = res["data"]
    if not data or type(data) ~= "table" then
        return nil, "data in response not a array"
    end
    return res["data"]
end

local scenes, err = list_scenes()
if scenes == nil then
    return utils.error_exit(500, err)
end

local resp = {
    code = 200,
    message = "success",
    data = { List = {} }
}
for _, v in ipairs(scenes) do
    local item = {
        SceneId = v["id"],
        SceneName = v["name"],
        Description = v["description"],
        Running = v["running"],
    }
    table.insert(resp.data.List, item)
end
ngx.say(json.encode(resp))
