# -*- coding: utf-8 -*-

import logging
import time
import lethingaccesssdk
from threading import Timer


class Light_Sensor(lethingaccesssdk.BaseThing):
    def __init__(self, config):
        super(Light_Sensor, self).__init__(config)
        self._illuminance = 200
        self._delta = 100

    @property
    def illuminance(self):
        illuminance = self._illuminance
        delta = self._delta
        if (illuminance >= 600 or illuminance <= 100):
            delta = -delta
        illuminance += delta
        self._delta = delta
        self._illuminance = illuminance
        return self._illuminance

    def callService(self, name, input):
        return 0, {}

    def getProperties(self, input):
        retDict = {}
        if 'MeasuredIlluminance' in input:
            retDict['MeasuredIlluminance'] = self._illuminance
        return 0, retDict

    def setProperties(self, input):
        logging.error("can't set value")
        return 100001, {}


def thing_behavior(device):
    device.online()
    while True:
        properties = {'MeasuredIlluminance': device.illuminance}
        device.reportProperties(properties)
        time.sleep(2)


try:
    thing_config = lethingaccesssdk.Config().getThingInfos()
    for config in thing_config:
        device = Light_Sensor(config)
        t = Timer(2, thing_behavior, (device, ))
        t.start()

except Exception as e:
    logging.error(e)


# don't remove this function
def handler(event, context):
    return 'hello world'
