#!/usr/bin/env python3
# -*- coding:-utf-8 -*-
import threading
import logging
try:
    import queue
except ImportError:
    import Queue as queue


class Thread(threading.Thread):
    def __init__(self, taskQueue):
        threading.Thread.__init__(self)
        self.taskQueue = taskQueue
        self.stopFlag = False
        self.setDaemon(True)
        self.start()

    def enableStopFlag(self):
        self.stopFlag = True

    def run(self):
        while (True):
            if (True == self.stopFlag):
                break
            try:
                logging.debug("getting ruleTask from dbusThreadPool queue...")
                func, kwargs = self.taskQueue.get(block=True, timeout=10)
                func(**kwargs)
            except queue.Empty:
                logging.debug("ThreadPool Queue is empty and have no ruleTask in the queue")
            except:
                logging.exception("ThreadPool: task exec err#####")


class ThreadPool(object):
    def __init__(self, threadNum, taskNum):
        self.threadList = []
        self.taskQueue = queue.Queue(maxsize=taskNum)
        self._init_threadPool(threadNum)

    def _init_threadPool(self, threadNum):
        for i in range(threadNum):
            thread = Thread(self.taskQueue)
            self.threadList.append(thread)

    def addTask(self, func, **kwargs):
        try:
            self.taskQueue.put((func, kwargs), block=False)
        except queue.Full:
            logging.warning("ThreaPool Queue is overflowed")
            pass

    def releasThread(self):
        for item in self.threadList:
            if (item.isAlive()):
                item.enableStopFlag()
                item.join()


def createThread(task, args=()):
    t = threading.Thread(target=task, args=args)
    t.setDaemon(True)
    t.start()
