#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# Author: jys
# File: timeSerialApp
# Time: 2019/10/11
# ====#====#====#====


from store import Tsdb_Store
import monotonic
import threading
import thread
import logging
import time
import os

_logger = logging.getLogger(__name__)

def task_tsStore(tsa):
	while (1):
		# 判断是否到存储时间
		if (False == tsa.isTimeOut()):
			_logger.info("looping...")
			time.sleep(1)
			continue
		
		# 存储属性\事件数据
		with tsa.lock:
			tsa.tsdbObj.storeProperty(tsa.propertyList)
			tsa.propertyList = []
			
			tsa.tsdbObj.storeEvent(tsa.eventList)
			tsa.eventList = []
		
		# 更新下次存储时间
		tsa.updateTimeOut()


class TimeSerialApp(object):
	def __init__(self):
		self.eventList = []
		self.propertyList = []
		self.timeOut = monotonic.monotonic() + 3
		self.lock = threading.Lock()
		
		host = os.environ.get('host', 'localhost')
		port = os.environ.get('port', 8242)

		# 创建tsdb 的句柄
		self.tsdbObj = Tsdb_Store(host, port)
		
		thread.createThread(task_tsStore, (self,))
	
	def isTimeOut(self):
		'''
			判断是否到存储时间
		:return:
		'''
		
		return True if (self.timeOut < monotonic.monotonic()) else False
	
	def updateTimeOut(self):
		'''
			更新存储时间
		:return:
		'''
		self.timeOut = monotonic.monotonic() + 3
	
	def pushEventInfo(self, pk, dn, eventName, eventData):
		'''
			将事件数据以结构化的形式进行缓存
		:param pk:
		:param dn:
		:param eventName:
		:param eventData:
		:return:
		'''
		dataDict = {
			"pk": pk,
			"dn": dn,
			"eventName": eventName,
			"eventData": eventData
		}
		
		with self.lock:
			self.eventList.append(dataDict)
	
	def pushPropertyInfo(self, pk, dn, propertyName, propertyData):
		'''
			将属性数据以结构化的形式进行缓存
		:param pk:
		:param dn:
		:param propertyName:
		:param propertyData:
		:return:
		'''
		dataDict = {
			"pk": pk,
			"dn": dn,
			"propertyName": propertyName,
			"propertyData": propertyData
		}
		
		with self.lock:
			self.propertyList.append(dataDict)
